@@ -28,7 +28,9 @@ import re, copy, os
 import Pyblio.Help
 from types import *
 
-from Pyblio import Config, Open, Utils, Key, Iterator, Selection, Autoload
+from Pyblio import Autoload, Config, Iterator, Key, Open, Selection, Utils
+
+from shutil import copyfile
 
 class Entry:
     '''
@@ -165,7 +167,7 @@ class DataBase:
     class Key. '''
 
     properties = {}
-
+    filemeta = {}
     id = 'VirtualDB'
 
     def __init__ (self, url):
@@ -173,6 +175,8 @@ class DataBase:
 	self.key = url
 
 	self.dict = {}
+
+	self.file_metadata = {}
 	return
 
 
@@ -278,7 +282,6 @@ class DataBase:
 	''' Returns an iterator for that database '''
 	return Iterator.DBIterator (self)
 
-
     def update (self, sorting = None):
 	''' Updates the Entries stored in the database '''
 	
@@ -287,20 +290,42 @@ class DataBase:
 
 	name = self.key.url [2]
 
-        # create a temporary file for the new version
-        tmp = os.path.join (os.path.dirname (name),
-                            '.#' + os.path.basename (name))
-        
-        tmpfile = open (tmp, 'w')
+        if Config.get ('base/directsave').data:
+            if Config.get ('base/backup').data:
+                copyfile (name, name + '.bak')
+
+            namefile = open (name, 'w')
+
+            iterator = Selection.Selection (sort = sorting).iterator (self.iterator ())
+	    Open.bibwrite (iterator, out = namefile, how = self.id, database=self)
+
+            namefile.close ()
+
+        else:
+
+            # create a temporary file for the new version
+            tmp = os.path.join (os.path.dirname (name),
+                                '.#' + os.path.basename (name))
+
+            tmpfile = open (tmp, 'w')
+
+            iterator = Selection.Selection (sort = sorting).iterator (self.iterator ())
+	    Open.bibwrite (iterator, out = tmpfile, how = self.id, database=self)
+
+	    tmpfile.close ()
+
+	    # if we succeeded, and backup is set, backup file
+            if Config.get ('base/backup').data:
+                os.rename (name, name + '.bak')
+
+	    # ...and bring new version online
+            os.rename (tmp, name)
 
-        iterator = Selection.Selection (sort = sorting).iterator (self.iterator ())
-	Open.bibwrite (iterator, out = tmpfile, how = self.id)
-        
-	tmpfile.close ()
-        
-	# if we succeeded, backup file
-	os.rename (name, name + '.bak')
-	# ...and bring new version online
-        os.rename (tmp, name)
         return
 
+
+    def get_metadata (self, key, default=None):
+	return self.file_metadata.get (key, default)
+    
+    def set_metadata (self, key, value):
+	self.file_metadata [key] = value
