@@ -37,8 +37,8 @@ _base_fieldtype = {
     LongText    : 0,
     Date        : 3,
     AuthorGroup : 1,
-    URL         : 0,
-    Reference   : 0,
+    URL         : 4,
+    Reference   : 4,
     }
 
 _text_fieldtype = Config.get ('bibtex+/capitalize').data
@@ -178,17 +178,12 @@ class Entry (Base.Entry):
         if isinstance (value, Date): return
         
         # then, convert as bibtex.
-        quote = 1
         if isinstance (value, Reference):
             value = string.join (map (lambda item: item.key, value.list), ', ')
-            quote = 0
             
-        if isinstance (value, URL):
-            quote = 0
-        
 	self.dict [key] = _bibtex.reverse (_fieldtype (Types.get_field (key)),
                                            Config.get ('bibtex+/braces').data,
-                                           value, quote)
+                                           value)
 	return
 
 
@@ -310,9 +305,12 @@ class Entry (Base.Entry):
 
 class BibtexIterator (Iterator.Iterator):
 
+
+
     def __init__ (self, db, parser):
 	self.db     = db
 	self.parser = parser
+	self.preamble = []
 	return
 
     def first (self):
@@ -320,24 +318,41 @@ class BibtexIterator (Iterator.Iterator):
 	return self.next ()
 
     def next (self):
-        try:
-            retval = _bibtex.next (self.parser)
-        except IOError, error:
-            raise Exceptions.ParserError ((str (error),))
-        
-	if retval == None: return None
 
-	name, fieldtype, offset, line, object = retval
+	while True:
+	    try:
+		retval = _bibtex.next_unfiltered (self.parser)
+	    except IOError, error:
+		raise Exceptions.ParserError ((str (error),))
+
+	    if retval == None:
+		self.db.set_metadata ('bibtex-preamble', self.preamble)
+		return None
+
+	    elif retval [0] == 'entry':
+		retval = retval [1]
+		name, fieldtype, offset, line, object = retval
+
+		if name:
+		    key = Key.Key (self.db, name)
+		else:
+		    key = None
+
+		fieldtype  = Types.get_entry (fieldtype)
+		entry = Entry (key, fieldtype, object, self.parser, line)
+
+		return entry
+
+	    elif retval [0] == 'preamble':
+		self.preamble.append (retval [1])
+		continue
+
+	    else :
+##		print '>>>IGNORED:', retval
+		continue
+	
 
-        if name:
-            key = Key.Key (self.db, name)
-        else:
-            key = None
-        
-	fieldtype  = Types.get_entry (fieldtype)
-	entry = Entry (key, fieldtype, object, self.parser, line)
 
-	return entry
 	
 		
 class DataBase (Base.DataBase):
@@ -348,6 +363,7 @@ class DataBase (Base.DataBase):
 	''' Initialisation '''
 
 	Base.DataBase.__init__ (self, basename)
+	self.prologue = None
 	self.__parsefile__ ()
 	return
 
@@ -371,7 +387,7 @@ class DataBase (Base.DataBase):
 		_bibtex.set_string (self.parser, k,
 				    _bibtex.reverse (_base_fieldtype [Text],
                                                      Config.get ('bibtex+/braces').data,
-						     user [k] [0], 0))
+						     user [k] [0]))
 
 	finished = 0
 	errors = []
@@ -409,7 +425,7 @@ class DataBase (Base.DataBase):
             else:
                 if self.dict.has_key (entry.key):
                     errors.append (_("%s:%d: key `%s' already defined") % (
-                        str (self.key), entry.line, entry.key.key))
+                        repr (self.key), entry.line, repr (entry.key.key)))
                 else:
                     self.dict [entry.key] = entry
 
@@ -429,7 +445,7 @@ class DataBase (Base.DataBase):
 		_bibtex.set_string (self.parser, k,
 				    _bibtex.reverse (_base_fieldtype [Text],
                                                      Config.get ('bibtex+/braces').data,
-						     user [k] [0], 0))
+						     user [k] [0]))
 	return
 
 
@@ -469,14 +485,16 @@ class DataBase (Base.DataBase):
     def new_entry (self, type):
         return Entry (None, type, {}, self.parser, 1)
     
-
+    def set_preamble (self, preamble):
+	self.set_metadata ('bibtex-preamble', preamble)
+	print '>>>> Preamble.', preamble
+	
 # ==================================================
 
 def _nativify (field, fieldtype):
     ''' private method to convert from field to native format '''
 
-    obj = _bibtex.reverse (fieldtype, Config.get ('bibtex+/braces').data,
-                           field, 1)
+    obj = _bibtex.reverse (fieldtype, Config.get ('bibtex+/braces').data, field)
     return _bibtex.get_native (obj)
 
 
@@ -587,7 +605,7 @@ def entry_write (entry, output):
     return
 
 
-def writer (it, output):
+def writer (it, output, preamble=[]):
     ''' Outputs all the items refered by the iterator <it> to the
     <stdout> stream '''
 
@@ -595,6 +613,10 @@ def writer (it, output):
     if Config.get ('base/advertise').data:
         output.write ('@comment{This file has been generated by Pybliographer}\n\n')
 
+    if preamble:
+	output.write ('@preamble {')
+	output.write (preamble)
+	output.write ('}\n\n')
     # locate the entries that belong to a BibTeX database
     # and those who contain crossreferences
     field_table = Config.get ('base/fields').data
@@ -617,6 +639,11 @@ def writer (it, output):
             
 	entry = it.next ()
 
+    # write the preamble, if any
+    if preamble:
+	for i in preamble:
+	    output.write ('@Preamble {%s}\n' % i)
+	output.write ('\n')
     # write the string definitions corresponding to these entries
     if len (header) > 0:
 	user = Config.get ('bibtex/macros').data
@@ -681,3 +708,9 @@ Autoload.register ('format', 'BibTeX', {'open'  : opener,
 					'write' : writer,
 					'iter'  : iterator})
 
+
+### Local Variables:
+### Mode: python
+### py-master-file : "ut_bibtex.py"
+### End:
+
