@@ -23,6 +23,8 @@
 
 ''' This module defines a Document class '''
 
+import gobject
+
 from gnome import ui
 
 import gnome
@@ -31,30 +33,181 @@ import gtk.glade
 
 from gtk import gdk
 
-from Pyblio.GnomeUI import Index, Entry, Utils, FileSelector, Editor
-from Pyblio.GnomeUI import Search, Format
+from Pyblio.GnomeUI import Editor, Entry, FileSelector, Format
+from Pyblio.GnomeUI import Index, OpenURL, Search, Utils
 from Pyblio.GnomeUI.Sort import SortDialog
 from Pyblio.GnomeUI.Medline import MedlineUI
 
-from Pyblio import Connector, Open, Exceptions, Selection, Sort, Base, Config
-from Pyblio import version, Fields, Types, Query
+from Pyblio import Base, Config, Connector, Exceptions, Fields, Open
+from Pyblio import Query, Resource, Selection, Sort, Types, version
+from Pyblio import Search as SearchCls
 
 import Pyblio.Style.Utils
 
 import os, string, copy, types, sys, traceback, stat
 
-import cPickle
-
-pickle = cPickle
-del cPickle
+import cPickle as pickle
 
 printable = string.lowercase + string.uppercase + string.digits
 
+
+
+
+
+
+uim_content = '''
+<ui>
+    <menubar name="Menubar">
+        <menu action="File">
+             <menuitem action="New"/>
+             <menuitem action="Open"/>
+             <menuitem action="Open_URL"/>
+             <menuitem action="Merge"/>
+             <menuitem action="Medline"/>
+             <menuitem action="Save"/>
+             <menuitem action="Save_As"/>
+             <separator/>
+             <menu action="Recent">
+                <placeholder name="Previous"/>
+             </menu>
+             <separator/>
+             <menuitem action="Close"/>
+             <menuitem action="Quit"/>
+        </menu>
+        <menu action="EditMenu">
+             <menuitem action="Cut"/>
+             <menuitem action="Copy"/>
+             <menuitem action="Paste"/>
+             <menuitem action="Clear"/>
+             <separator/>
+             <menuitem action="Add"/>
+             <menuitem action="Edit"/>
+             <menuitem action="Delete"/>
+             <separator/>
+             <menuitem action="Find"/>
+             <menuitem action="Sort"/>
+        </menu>
+	<menu action="ViewMenu">
+	     <menu action="ViewResource">
+                 <placeholder name="Viewables" />
+             </menu>
+        </menu>
+        <menu action="CiteMenu">
+             <menuitem action="Cite"/>
+             <menuitem action="Format"/>
+        </menu>
+        <menu action="Settings">
+             <menuitem action="Fields"/>
+             <menuitem action="Preferences"/>
+             <separator/>
+             <menuitem action="Forget"/>
+        </menu>
+        <menu action="HelpMenu">
+             <menuitem action="Contents"/>
+             <menuitem action="About"/>
+        </menu>
+    </menubar>
+
+    <toolbar name="Toolbar">
+        <toolitem action="Open"/>
+        <toolitem action="Save"/>
+        <separator/>
+        <toolitem action="Add"/>
+        <separator/>
+        <toolitem action="Cite"/>
+        <separator/>
+    </toolbar>
+    
+    <popup name="Popup">
+         <menuitem action="Add"/>
+         <menuitem action="Edit"/>
+	 <menu action="ViewResource">
+	     <placeholder name="Viewables" />
+         </menu>
+         <menuitem action="Delete"/>
+    </popup>
+</ui>
+'''
+
 class Document (Connector.Publisher):
     
     def __init__ (self, database):
 
+        self.uim = gtk.UIManager ()
 
+        self.recents = None
+        self.viewables = None
+	
+        self.actiongroup = gtk.ActionGroup ('Main')
+        
+        self.actiongroup.add_actions ([
+            # id     stock            label         accel   tooltip   callback
+            ('File', None,                _('_File')),
+            ('EditMenu', None,            _('_Edit')),
+	    ('ViewMenu', None,            _('_View')),
+            ('CiteMenu', None,            _('_Cite')),
+            ('Settings', None,            _('_Settings')),
+            ('HelpMenu', None,            _('_Help')),
+            ('Recent', None, _('Recent documents')),
+            
+            ('New',  gtk.STOCK_NEW,   None,         None,   None,     self.new_document),
+            ('Open', gtk.STOCK_OPEN,  None,         None,   _('Open a file'), self.ui_open_document),
+            ('Open_URL', None, _('Open _Location'), '<control>l',   None, self.ui_open_location),
+            ('Save', gtk.STOCK_SAVE,  None,         None,   _('Save the current file'), self.save_document),
+            ('Save_As', gtk.STOCK_SAVE_AS,  None,         None,   None,     self.save_document_as),
+            ('Close', gtk.STOCK_CLOSE,  None,         None,   None,     self.close_document),
+            ('Quit', gtk.STOCK_QUIT,  None,         None,   None,     self.exit_application),
+
+            ('Merge',   None, _('Merge With...'),    '<control>g',  None, self.merge_database),
+            ('Medline', None, _('Medline Query...'), '<control>m',  None, self.query_database),
+
+
+
+            ('Cut', gtk.STOCK_CUT,  None,         None,   None,     self.cut_entry),
+            ('Copy', gtk.STOCK_COPY,  None,         None,   None,     self.copy_entry),
+            ('Paste', gtk.STOCK_PASTE,  None,         None,   None,     self.paste_entry),
+            ('Clear', gtk.STOCK_CLEAR,  None,         None,   None,     self.clear_entries),
+            ('Add', gtk.STOCK_ADD,  None,   '<shift><control>n',  _('Add a new entry'), self.add_entry),
+            ('Delete', gtk.STOCK_DELETE,  None,         None,   None,     self.delete_entry),
+            ('Find', gtk.STOCK_FIND,  None,         None,   None,     self.find_entries),
+            
+            ('Sort', None, _('S_ort...'), None,  None, self.sort_entries),
+            ('Cite', gtk.STOCK_JUMP_TO,   _('Cite...'), None,  _('Cite key(s)'), self.lyx_cite),
+            ('Format', gtk.STOCK_EXECUTE, _('Format...'), None,  None, self.format_entries),
+
+            ('Fields', None, _('Fields...'), None,  None, self.set_fields),
+            ('Preferences', gtk.STOCK_PREFERENCES,  None,         None,   None,     self.set_preferences),
+            ('Forget', None, _('Forget all changes'),     None,   None,     self.forget_changes_cb),
+            
+            ('Contents', gtk.STOCK_HELP, None,   None,   None,     self.on_documentation),
+	    ('ViewResource', None, _('_Resource'), None, None, self.view_entry),
+            ])
+
+        if gtk.pygtk_version >= (2,6,0):
+            self.actiongroup.add_actions ([
+                ('About', gtk.STOCK_ABOUT, None,   None,   None,     self.about),
+                ('Edit', gtk.STOCK_EDIT,  None, '<shift><control>o',   None,     self.edit_entry),
+                ])
+
+        else:
+            self.actiongroup.add_actions ([
+                ('About', None, _('_About'),   None,   None,     self.about),
+                ('Edit', None,  _('_Edit'),    '<shift><control>o', None,    self.edit_entry),
+                ])
+            
+        prev = self.actiongroup.get_action ('Recent')
+        
+        prev.set_property ('is-important', True)
+        prev.set_property ('hide-if-empty', False)
+
+        view_action = self.actiongroup.get_action ('ViewResource')
+	view_action.set_property ('hide-if-empty', False)
+        self.uim.insert_action_group (self.actiongroup, 0)
+        self.uim.add_ui_from_string (uim_content)
+
+        self.uim.ensure_update ()
+
+        
         gp = os.path.join (version.pybdir, 'glade', 'pyblio.glade')
         
         self.xml = gtk.glade.XML (gp, 'main', domain = 'pybliographer')
@@ -63,17 +216,45 @@ class Document (Connector.Publisher):
         self.w = self.xml.get_widget ('main')
         self.paned = self.xml.get_widget ('main_pane')
 
+        self.w.set_menus (self.uim.get_widget ('/Menubar'))
+        self.w.set_toolbar (self.uim.get_widget ('/Toolbar'))
+
+        self.w.add_accel_group (self.uim.get_accel_group ())
+
         self.w.add_events (gdk.KEY_PRESS_MASK)
         
         self.w_save_btn = self.xml.get_widget ('_w_save_btn')
         self.w_save_mnu = self.xml.get_widget ('_w_save_mnu')
+
+        # We manually add a simple search area
+        t = self.uim.get_widget ('/Toolbar')
+        h = gtk.HBox()
+
+        i = gtk.Image()
+        i.set_from_stock(gtk.STOCK_FIND, gtk.ICON_SIZE_LARGE_TOOLBAR)
+        h.pack_start(i, False, False)
+
+        # create a tooltips object
+        self.toolbartips = gtk.Tooltips()
+
+        self.quick_search = gtk.Entry()
+        self.quick_search.connect('activate', self.simple_search)
+        h.pack_start(self.quick_search, False, False)
+        self.toolbartips.set_tip(self.quick_search, _('Quick search'))
+
+        i = gtk.ToolItem()
+        i.add(h)
+        t.insert(i, -1)
+        
+        i.show_all()
         
         # The Index list
-        self.index = Index.Index ()
+        self.index = Index.Index (popup = self.uim.get_widget ('/Popup'))
         self.paned.add1 (self.index.w)
         
         self.index.Subscribe ('new-entry',      self.add_entry)
         self.index.Subscribe ('edit-entry',     self.edit_entry)
+##      self.index.Subscribe ('view-entry',     self.view_entry)
         self.index.Subscribe ('delete-entry',   self.delete_entry)
         self.index.Subscribe ('select-entry',   self.update_display)
         self.index.Subscribe ('select-entries', self.freeze_display)
@@ -117,6 +298,9 @@ class Document (Connector.Publisher):
         
         self.modification_date = None
 
+        # for autosave
+        self.source_id = None
+
         # set the default sort method
         default = Utils.config.get_string ('/apps/pybliographic/sort/default')
         if default is not None: default = pickle.loads (default)
@@ -124,6 +308,7 @@ class Document (Connector.Publisher):
         self.sort_view (default)
 
         self._title_set ()
+        self._set_edit_actions (False)
         return
 
     def _title_set (self):
@@ -155,43 +340,39 @@ class Document (Connector.Publisher):
 
     def update_history (self, history):
         ''' fill the " Previous Documents " menu with the specified list of documents '''
-        sub = self.xml.get_widget ('previous_documents')
-        factory = gtk.ItemFactory (gtk.Menu, '<main>', None)
 
-        # This list contains the info needed to create the "Previous Documents" menu.
-        menuinfo = []
+        if self.recents:
+            for mid in self.recents_mid:
+                self.uim.remove_ui (mid)
+                
+            self.uim.remove_action_group (self.recents)
+
+        self.recents_mid = []
+        self.recents = gtk.ActionGroup ('Recent')
+
+        self.uim.insert_action_group (self.recents, 1)
 
-        # This list contains the names of the menu entries that can be
-        # fetched back by factory.get_widget calls. The two lists
-        # differ as the factory.create_items () function interprets
-        # underscores to mark shortcuts, whereas the get_widget call
-        # does not escape the underscores...
-        names = []
-        
         for item in history:
             # Display name in the menu
-            filename = string.replace (item [0], '/', '\/')
-            quoted   = string.replace (filename, '_', '__')
-            menuinfo.append (('/' + quoted, None,
-                              self._history_open_cb, 0, None))
-            names.append ('/' + filename)
+            quoted   = string.replace (item [0], '_', '__')
+        
+            mid = self.uim.new_merge_id ()
 
-        factory.create_items (menuinfo)
-
-        # Bind the actual file info to each menu entry
-        i = 0
-        for item in names:
-            w = factory.get_widget (item)
-            w.set_data ('file', history [i])
-            i = i + 1
+            self.recents_mid.append (mid)
             
-        sub.set_submenu (factory.get_widget ('<main>'))
-        return
+            action = gtk.Action (str (mid), quoted, None, None)
+            self.recents.add_action (action)
 
+            action.connect ('activate', self._history_open_cb, item)
+        
+            self.uim.add_ui (mid, '/Menubar/File/Recent', str (mid),
+                             str (mid), gtk.UI_MANAGER_MENUITEM, False)
+
+        return
 
     def _history_open_cb (self, id, w):
 
-        file, type = w.get_data ('file')
+        file, type = w
         
         if not self.confirm (): return
 
@@ -255,7 +436,7 @@ class Document (Connector.Publisher):
         if self.data.key is None:
             text = _("New database")
         else:
-            text = str (self.data.key)
+            text = self.data.key.get_url ()
 
         li = len (self.index)
         ld = len (self.data)
@@ -274,8 +455,10 @@ class Document (Connector.Publisher):
         if self.changed:
             text = text + ' ' + _("[modified]")
 
-        self.w_save_btn.set_sensitive (self.changed)
-        self.w_save_mnu.set_sensitive (self.changed)
+        if self.selection.search:
+            text += ' - ' + _('view limited to: %s') % self.selection_name
+
+        self.actiongroup.get_action ('Save').set_property ('sensitive', self.changed)
 
         self.statusbar.set_default (text)
         return
@@ -285,12 +468,34 @@ class Document (Connector.Publisher):
         ''' eventually ask for modification cancellation '''
         
         if self.changed:
-            return Utils.Callback (_("The database has been modified.\nDiscard changes ?"),
-                                   self.w).answer ()
-        
-        return 1
+	    if Config.get('gnome/old-confirmation-dialog').data:
+		return Utils.Callback (_("The database has been modified.\nDiscard changes?"),
+				       self.w).answer ()
+      
+	    else:
+		if Utils.Callback (_("The database has been modified.\nSave changes?"),
+				   self.w).answer () and self.modification_check ():
+		    self.save_document ()
+		else:
+		    return True
+	return 1
 
-        
+    def modification_check (self):
+	"""Check for external modification, if necessary,
+	ask user for permission to save.
+	Returns True if no modifications or overwrite accepted by user."""
+ 
+        if self.modification_date:
+            mod_date = os.stat (self.data.key.url [2]) [stat.ST_MTIME]
+            
+            if mod_date > self.modification_date:
+                return  Utils.Callback (
+		    _("The database has been externally modified.\nOverwrite changes ?"),
+		    self.w).answer ()
+	    
+	return True
+	
+      
     def new_document (self, * arg):
         ''' callback corresponding to the "New Document" button '''
         
@@ -321,7 +526,7 @@ class Document (Connector.Publisher):
         ''' add all the entries of another database to the current one '''
         # get a new file name
         (url, how) = FileSelector.URLFileSelection (_("Merge file"),
-                                                    url = True, has_auto = True).run ()
+                                                    has_auto = True).run ()
 
         if url is None: return
 
@@ -373,11 +578,43 @@ class Document (Connector.Publisher):
         self.open_document (url, how)
         return
 
+
+    def ui_open_location (self, * arg):
+        ''' callback corresponding to "Open Location" '''
+
+        if not self.confirm (): return
+
+        (url, how) = OpenURL.OpenDialog (self.w).run ()
+
+        if url == None or url == "": return
+        self.open_document (url, how)
+        return
     
+
     def open_document (self, url, how = None, no_name = False):
 
         Utils.set_cursor (self.w, 'clock')
-        
+
+        orig_url = Fields.URL (url)
+        url = orig_url.get_url ()
+
+        restore = False
+
+        if orig_url.url [0] == 'file':
+
+            name = orig_url.url [2]
+            auto_save = os.path.join (os.path.dirname (name),
+                            'x-pyblio-save-' + os.path.basename (name))
+
+            if os.path.exists (auto_save):
+                mod_date = os.stat (name) [stat.ST_MTIME]
+                mod_date_auto = os.stat (auto_save) [stat.ST_MTIME]
+                if mod_date < mod_date_auto:
+                    restore = Utils.Callback (_("An autosave file was found which is newer than the original file.\nDo you want to restore it?"), self.w).answer ()
+
+                    if restore: url = auto_save
+
+
         try:
             data = Open.bibopen (url, how = how)
             
@@ -390,19 +627,85 @@ class Document (Connector.Publisher):
                                 parent = self.w)
             return
 
+
+        # remove the old autosave object
+        if self.data.key is not None and self.source_id:
+            gobject.source_remove (self.source_id)
+
+        # remove old autosave file if exists
+        if self.data.key:
+            if self.data.key.url [0] == 'file':
+                old_file = self.data.key.url [2]
+                old_auto_save = os.path.join (os.path.dirname (old_file),
+                                'x-pyblio-save-' + os.path.basename (old_file))
+
+                if os.path.exists (old_auto_save):
+                    try:
+                        os.remove (old_auto_save)
+                    except (OSError, IOError), error:
+                        Utils.set_cursor (self.w, 'normal')
+                        self.w.error (_("Unable to remove autosave file `%s':\n%s") % (str (old_auto_save), str (error)))
+                        return
+
+
         Utils.set_cursor (self.w, 'normal')
 
         if no_name: data.key = None
         
         self.data    = data
-        self.redisplay_index (0)
+
+
+        if restore:
+
+            # restore the original url internally,
+            # and change the document status
+            self.data.key = orig_url
+            self.redisplay_index (1)
+
+        else:
+            self.redisplay_index (0)
         
+
         self._title_set ()
 
         # eventually warn interested objects
         self.issue ('open-document', self)
+
+        # create autosave object if needed
+        if Config.get ('base/autosave').data:
+    	    savetimeout = Config.get ('base/autosave interval').data
+            self.source_id = gobject.timeout_add (savetimeout * 60 * 1000, self.autosave, url, self.data.id)
+
         return
 
+
+    def autosave (self, url, how):
+        ''' autosave file as x-pyblio-save-filename '''
+
+        if self.data.key.url [0] != 'file': return False
+
+        name = self.data.key.url [2]
+
+        # create an autosave file
+        save = os.path.join (os.path.dirname (name),
+                            'x-pyblio-save-' + os.path.basename (name))
+
+        if self.changed:
+
+            try:
+                savefile = open (save, 'w')
+            except (IOError, OSError), error:
+                self.w.error (_("Error during autosaving:\n%s") % error [1])
+                return False
+
+            iterator = Selection.Selection (sort = self.selection.sort)
+            Open.bibwrite (iterator.iterator (self.data.iterator ()),
+                           out = savefile, how = how, database=self.data)
+
+            savefile.close ()
+
+        return True
+
     
     def save_document (self, * arg):
         if self.data.key is None:
@@ -411,14 +714,9 @@ class Document (Connector.Publisher):
 
         file = self.data.key.url [2]
         
-        if self.modification_date:
-            mod_date = os.stat (file) [stat.ST_MTIME]
-            
-            if mod_date > self.modification_date:
-                if not Utils.Callback (_("The database has been externally modified.\nOverwrite changes ?"),
-                                       self.w).answer ():
-                    return
-        
+	if not self.modification_check ():
+	    return
+	
         Utils.set_cursor (self.w, 'clock')
         try:
             try:
@@ -447,14 +745,15 @@ class Document (Connector.Publisher):
     
     def save_document_as (self, * arg):
         # get a new file name
-        (url, how) = FileSelector.URLFileSelection (_("Save As..."),
-                                                    url = False, has_auto = False, is_save = True).run ()
+        (url, how) = FileSelector.URLFileSelection (
+	    _("Save As..."), has_auto = False, is_save = True).run ()
         
         if url is None: return
 
         if os.path.exists (url):
-            if not Utils.Callback (_("The file `%s' already exists.\nOverwrite it ?")
-                                   % url, parent = self.w).answer ():
+            if not Utils.Callback (
+		_("The file `%s' already exists.\nOverwrite it ?")
+		% url, parent = self.w).answer ():
                 return
 
         try:
@@ -467,8 +766,28 @@ class Document (Connector.Publisher):
 
         iterator = Selection.Selection (sort = self.selection.sort)
         Open.bibwrite (iterator.iterator (self.data.iterator ()),
-                       out = file, how = how)
+                       out = file, how = how, database=self.data)
         file.close ()
+
+        # remove the old autosave object
+        if self.data.key is not None and self.source_id:
+            gobject.source_remove (self.source_id)
+
+        # remove old autosave file
+        if self.data.key:
+            if self.data.key.url [0] == 'file':
+                old_file = self.data.key.url [2]
+                old_auto_save = os.path.join (os.path.dirname (old_file),
+                                'x-pyblio-save-' + os.path.basename (old_file))
+
+                if os.path.exists (old_auto_save):
+                    try:
+                        os.remove (old_auto_save)
+                    except (OSError, IOError), error:
+                        Utils.set_cursor (self.w, 'normal')
+                        self.w.error (_("Unable to remove autosave file `%s':\n%s") % (str (old_auto_save), str (error)))
+                        return
+
         
         try:
             self.data = Open.bibopen (url, how = how)
@@ -490,6 +809,12 @@ class Document (Connector.Publisher):
         Utils.set_cursor (self.w, 'normal')
 
         self.update_status (0)
+
+        # create the new autosave object if needed
+        if Config.get ('base/autosave').data:
+            savetimeout = Config.get ('base/autosave interval').data
+            self.source_id = gobject.timeout_add (savetimeout * 60 * 1000, self.autosave, url, self.data.id)
+
         return
 
     
@@ -503,7 +828,27 @@ class Document (Connector.Publisher):
 
 
     def close_document_request (self):
-        return self.confirm ()
+        answer = self.confirm ()
+        # remove autosave object with closing
+        if answer and self.source_id:
+            gobject.source_remove (self.source_id)
+
+        # remove old autosave file
+        if answer and self.data.key:
+            if self.data.key.url [0] == 'file':
+                old_file = self.data.key.url [2]
+                old_auto_save = os.path.join (os.path.dirname (old_file),
+                                'x-pyblio-save-' + os.path.basename (old_file))
+
+                if os.path.exists (old_auto_save):
+                    try:
+                        os.remove (old_auto_save)
+                    except (OSError, IOError), error:
+                        Utils.set_cursor (self.w, 'normal')
+                        self.w.error (_("Unable to remove autosave file `%s':\n%s") % (str (old_auto_save), str (error)))
+                        return
+
+        return answer
 
     
     def exit_application (self, * arg):
@@ -624,7 +969,15 @@ class Document (Connector.Publisher):
         self.index.select_item (new)
         return
     
-    
+
+    def view_entry (self, action, *item):
+	if item:
+	    entry, key, url, value = item [0]
+##	    print 'VIEW ENTRY:', entry, key, url, value
+	    Resource.StartViewer (entry, key, value, parent=self.w, document=self)
+	else: #print 'Call to VIEW ENTRY ignored'
+	    return
+
     def delete_entry (self, * arg):
         ''' removes the selected list of items after confirmation '''
         entries = self.index.selection ()
@@ -648,7 +1001,19 @@ class Document (Connector.Publisher):
         self.redisplay_index (1)
         self.index.select_item (offset)
         return
-    
+
+    def simple_search(self, w, *arg):
+        q = w.get_text().strip()
+        if q:
+            try:
+                test = SearchCls.AnyTester(q.encode('latin-1'))
+            except UnicodeEncodeError:
+                self.w.error (_("your search text must contain\nlatin-1 characters only"))
+                return
+        else:
+            test = None
+
+        self.limit_view(q, test)
     
     def find_entries (self, * arg):
         if self.search_dg is None:
@@ -659,8 +1024,9 @@ class Document (Connector.Publisher):
         return
 
 
-    def limit_view (self, search):
+    def limit_view (self, name, search):
         self.selection.search = search
+        self.selection_name = name
         self.redisplay_index ()
         return
 
@@ -721,7 +1087,6 @@ class Document (Connector.Publisher):
             try:
                 self.lyx = LyX.LyXClient ()
             except IOError, msg:
-                msg = msg [1].decode (enc)
                 self.w.error (_("Can't connect to LyX:\n%s") % msg)
                 return
 
@@ -734,19 +1099,61 @@ class Document (Connector.Publisher):
         return
     
 
+    def _set_edit_actions (self, value):
+        for action in ('Cite', 'Copy', 'Cut', 'Delete', 'Edit', 'ViewResource'):
+            self.actiongroup.get_action (action).set_property ('sensitive', value)
+        return
+        
     def update_display (self, entry):
-        self.display.display (entry)
+        if entry:
+            self.display.display (entry)
+	    self.update_viewables (entry)
+        self._set_edit_actions (entry is not None)
         return
 
     
     def freeze_display (self, entry):
         self.display.clear ()
+        self._set_edit_actions (True)
         return
 
+    def update_viewables (self, entry):
 
+	if self.viewables:
+	    for item in self.viewables_id:
+		self.uim.remove_ui (item)
+	    self.uim.remove_action_group (self.viewables)
+
+	self.viewables_id = []
+	self.viewables = gtk.ActionGroup ('Viewables')
+	self.uim.insert_action_group (self.viewables, 1)
+	
+	viewables = Resource.get_viewables (entry)
+	for key, url, value in viewables:
+## 	    text = u'<span foreground="BLUE" weight="bold">%s</span>   %s' %(
+## 		key.upper (), value)
+	    text = u"%s   %s" % (key.upper (), value)
+	    mergeid = self.uim.new_merge_id ()
+	    self.viewables_id.append (mergeid)
+	    action = gtk.Action (str(mergeid), text, None, None)
+	    self.viewables.add_action (action)
+	    action.connect ('activate', self.view_entry, (entry, key, url, value))
+
+	    self.uim.add_ui (mergeid, '/Menubar/ViewMenu/ViewResource', str(mergeid), 
+			     str(mergeid), gtk.UI_MANAGER_MENUITEM, False)
+	    self.uim.add_ui (mergeid, '/Popup/ViewResource', str(mergeid), 
+			     str(mergeid), gtk.UI_MANAGER_MENUITEM, False)
+	return
+    
     def key_pressed (self, app, event):
 
         # filter out special keys
+        
+        if event.keyval == gtk.keysyms.Escape:
+            # the Esc key restores view to "all entries"
+            self.limit_view (None, None)
+            self.quick_search.set_text('')
+            
         if (event.string < 'a' or event.string > 'z') and \
            (event.string < '0' or event.string > '9'): return False
 
@@ -800,7 +1207,7 @@ class Document (Connector.Publisher):
         import gobject
 
         try:
-            gnome.help_display ('pybliographer', 'getting-started')
+            gnome.help_display ('pybliographer', None)
             
         except gobject.GError, msg:
             self.w.error (_("Can't display documentation:\n%s") % msg)
@@ -815,6 +1222,7 @@ class Document (Connector.Publisher):
                           _("Gnome interface to the Pybliographer system."),
                           ['Hervé Dréau',
                            'Frédéric Gobry',
+                           'Zoltán Kóta',
                            'Travis Oliphant',
                            'Darrell Rudmann',
                            'Peter Schulte-Stracke',
