@@ -23,17 +23,13 @@
 #   entry editor for more than 50 chars
 # added: Note taking widget
 
-import re, string
+import copy, gobject, gtk, re, string
 from gnome import ui
-import gtk
-
-import copy, re
 
 from Pyblio import Base, Config, Connector, Exceptions, Fields, Key, Types
+from Pyblio.GnomeUI import Common, Compat, FieldsInfo, FileSelector, Mime, Utils
 
-from Pyblio.GnomeUI import FieldsInfo, Mime, Utils
-
-key_re = re.compile ("^[\w:_+-.()]+$")
+key_re = re.compile ("^[\w:_+-.()/]+$")
 
 _newcontent = {
     Fields.AuthorGroup : _("Last Name, First Name"),
@@ -356,7 +352,7 @@ class Date (BaseField):
         if text != '':
             try: day = int (text)
             except ValueError:
-                ui.gnome_error_dialog_parented (_("Invalid day field in date"),
+                Compat.error_dialog_parented (_("Invalid day field in date"),
                                                 self.day.get_toplevel ())
                 return -1
         
@@ -364,7 +360,7 @@ class Date (BaseField):
         if text != '':
             try: month = int (text)
             except ValueError, err:
-                ui.gnome_error_dialog_parented (_("Invalid month field in date"),
+                Compat.error_dialog_parented (_("Invalid month field in date"),
                                                 self.day.get_toplevel ())
                 return -1
         
@@ -372,7 +368,7 @@ class Date (BaseField):
         if text != '':
             try: year = int (text)
             except ValueError: 
-                ui.gnome_error_dialog_parented (_("Invalid year field in date"),
+                Compat.error_dialog_parented (_("Invalid year field in date"),
                                                 self.day.get_toplevel ())
                 return -1
         
@@ -385,7 +381,7 @@ class Date (BaseField):
         try:
             entry [self.field] = Fields.Date ((year, month, day))
         except Exceptions.DateError, error:
-            ui.gnome_error_dialog_parented (str (error),
+            Compat.error_dialog_parented (str (error),
                                             self.day.get_toplevel ())
             return -1
         return 1
@@ -452,7 +448,9 @@ class Reference (BaseField):
         self.initial = self.current
         return
     
+
     def drag_received (self, *arg):
+
         selection = arg [4]
         info      = arg [5]
 
@@ -489,19 +487,47 @@ class Reference (BaseField):
 class URL (BaseField):
     
     def create_widget (self, h):
-        self.edit = gtk.Entry ()
-        self.edit.set_editable (True)
-        self.edit.set_text (self.string.decode ('latin-1'))
-        self.edit.show ()
+## 	self.url = self.value
+## 	self.edit = Common.filechooserbutton(self.url, 'enter/edit',)
+##      self.edit.show ()
+##      h.pack_start (self.edit)
+	self.newurl = None
 
-        h.pack_start (self.edit)
+	self.box = gtk.HBox (spacing=6)
+	self.edit = gtk.Entry ()
+	self.edit.set_editable (True)
+	self.old_url = self.string.decode ('latin-1')
+	self.edit.set_text (self.old_url)
+	self.box.pack_start (self.edit)
+	self.button = gtk.Button (_('Browse...'))
+	self.button.connect ("clicked", self.cb_clicked)
+	self.box.pack_start (self.button, False)
+	h.pack_start (self.box)
         return 0
-
+  
+##    def update (self, entry):
+	
+##	self.newurl = self.edit.get_url ()
+	
+## 	if self.newurl:
+## 	    entry [self.field] = self.newurl
+## 	    return 1
+## 	else:
+## 	    return 0
+	
 
     def update_content (self, entry, text):
         entry [self.field] = Fields.URL (text)
         return
 
+    def cb_clicked (self, *args):
+	(url, how) = FileSelector.URLFileSelection (
+	    _("Select file"), show_type = False).run ()
+	if url:
+	    self.edit.set_text (url)
+	return 
+	
+
 
 class RealEditor (Connector.Publisher):
     ''' Edits in standard form '''
@@ -534,12 +560,19 @@ class RealEditor (Connector.Publisher):
         table.attach (self.key,
                       1, 2, 1, 2, yoptions = 0)
 
-        self.menu = gtk.OptionMenu ()
-        menu = gtk.Menu ()
-        self.menu.set_menu (menu)
+        # The list store will hold both the identifier and the type of
+        # each entry
+        liststore = gtk.ListStore (gobject.TYPE_STRING,
+                                   gobject.TYPE_PYOBJECT)
 
-        table.attach (self.menu,
-                      0, 1, 1, 2, yoptions = 0)
+        self.menu = gtk.ComboBox (liststore)
+
+        cell = gtk.CellRendererText()
+
+        self.menu.pack_start (cell, True)
+        self.menu.add_attribute (cell, 'text', 0)
+
+        table.attach (self.menu, 0, 1, 1, 2, yoptions = 0)
 
         entry_list = Config.get ("base/entries").data.values ()
         entry_list.sort (lambda x, y: cmp (x.name, y.name))
@@ -548,11 +581,12 @@ class RealEditor (Connector.Publisher):
         history = 0
         for entry in entry_list:
             if entry == self.entry.type: history = i
-            Utils.popup_add (menu, entry.name,
-                             self.menu_select, entry)
+            liststore.append ((entry.name, entry))
+            
             i = i + 1
 
-        self.menu.set_history (history)
+        self.menu.set_active (history)
+        self.menu.connect ("changed", self.menu_select)
         
         table.show_all ()
         self.w.pack_start (table, False, False)
@@ -593,7 +627,10 @@ class RealEditor (Connector.Publisher):
         return
 
 
-    def menu_select (self, menu, entry):
+    def menu_select (self, menu):
+        idx   = menu.get_active_iter ()
+        entry = menu.get_model ().get_value (idx, 1)
+        
         # update the current entry
         new = self.update (self.database, copy.deepcopy (self.entry))
         if new is None:
@@ -866,7 +903,7 @@ class RealEditor (Connector.Publisher):
             modified = True
         else:
             if not key_re.match (key):
-                ui.gnome_error_dialog_parented (
+                Compat.error_dialog_parented (
                     _("Invalid key format"), self.w.get_toplevel ())
                 return None
 
@@ -874,7 +911,7 @@ class RealEditor (Connector.Publisher):
 
             if key != self.entry.key:
                 if database.has_key (key):
-                     ui.gnome_error_dialog_parented (
+                     Compat.error_dialog_parented (
                          _("Key `%s' already exists") % str (key.key),
                          self.w.get_toplevel ())
                      return None
@@ -891,7 +928,7 @@ class RealEditor (Connector.Publisher):
             except UnicodeError:
                 f = Types.get_field (item.field)
                 
-                ui.gnome_error_dialog_parented (
+                Compat.error_dialog_parented (
                     _("The `%s' field contains a non Latin-1 symbol") %
                     f.name, self.w.get_toplevel ())
                 return None
@@ -961,7 +998,7 @@ class NativeEditor (Connector.Publisher):
             text = text.encode ('latin-1')
 
         except UnicodeError:
-            ui.gnome_error_dialog_parented (
+            Compat.error_dialog_parented (
                 _("Your text contains non Latin-1 symbols"),
                 self.w.get_toplevel ())
             return None
@@ -1350,23 +1387,23 @@ class  LT_Dialog_1     :
             gtk.Label (
             _('Name of the new annotation:')),
             True, True, 6)
-        menu = gtk.Menu()
-        self.options = gtk.OptionMenu()
-        self.options.set_menu(menu)
+        
+        self.options = gtk.combo_box_new_text ()
+
         self.dialog.vbox.pack_start (self.options, True, True, 12)
         self.fields = [ x for x in Config.get ('base/fields').data
                         if Types.get_field(x).type == Fields.LongText]
         self.fields.sort()
         for i in self.fields:
-            menu.append(gtk.MenuItem(i))
-        self.options.set_history(0)
-        self.options.grab_focus()
+            self.options.append_text (i)
+        self.options.set_active (0)
+
         self.options.connect ('changed', self.changed)
         self.dialog.set_default_response(gtk.RESPONSE_ACCEPT)
         self.value = self.fields[0]
 
     def changed (self, *args):
-        val = self.options.get_history()
+        val = self.options.get_active ()
         self.value = self.fields[val]
 
     def run (self):
