@@ -33,14 +33,14 @@ class LyXClient:
         pin = os.path.expanduser (base + '.in')
         try:
             ans = os.stat (pin)
-        except os.error:
-            raise IOError, (-1, _("no input pipe `%s'") % pin)
+        except OSError, msg:
+            raise IOError (_("no input pipe `%s'") % pin)
         
         pout = os.path.expanduser (base + '.out')
         try:
             ans = os.stat (pout)
         except os.error:
-            raise IOError, (-1, _("no output pipe `%s'") % pout)
+            raise IOError (_("no output pipe `%s'") % pout)
 
         def noaction (* arg): return
         
